/*
 * This file is a part of morm
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2018, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */


#include "baseescaper.h"
#include "utf8/utf8.h"


namespace morm
{

BaseEscaper::~BaseEscaper()
{
}


void BaseEscaper::esc(const std::wstring & val, PT::TextStream & stream)
{
	char utf8_buf[10];

	for(size_t i = 0 ; i < val.size() ; ++i)
	{
		size_t utf8_len = PT::IntToUTF8((int)val[i], utf8_buf, sizeof(utf8_buf));

		for(size_t a = 0 ; a < utf8_len ; ++a)
		{
			esc(utf8_buf[a], stream);
		}
	}
}


void BaseEscaper::esc(const wchar_t * val, PT::TextStream & stream)
{
	char utf8_buf[10];

	for(size_t i = 0 ; val[i] != 0 ; ++i)
	{
		size_t utf8_len = PT::IntToUTF8((int)val[i], utf8_buf, sizeof(utf8_buf));

		for(size_t a = 0 ; a < utf8_len ; ++a)
		{
			esc(utf8_buf[a], stream);
		}
	}
}


void BaseEscaper::esc(const std::string & val, PT::TextStream & stream)
{
	for(size_t i = 0 ; i < val.size() ; ++i)
	{
		esc(val[i], stream);
	}
}



}

