/*
 * This file is a part of morm
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2018, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef headerfile_morm_modelconnector
#define headerfile_morm_modelconnector

#include "expression.h"



namespace morm
{
class Model;


class ModelConnector
{

public:

	ModelConnector();
	~ModelConnector();

	// FIX ME
	// add c-copy ctr (allocate a new stream and expression)

	void set_expression(Expression & expression);
	void set_stream(PT::TextStream & stream);

	Expression * get_expression();
	PT::TextStream * get_stream();

	virtual void to_json(PT::TextStream & stream, Model & model);
	virtual void insert(PT::TextStream & stream, Model & model, int db_type);
	virtual void update(PT::TextStream & stream, Model & model, int db_type);

	virtual void insert(PT::TextStream & stream, Model & model);
	virtual void update(PT::TextStream & stream, Model & model);

	virtual void to_json(Model & model);
	virtual void insert(Model & model, int db_type);
	virtual void update(Model & model, int db_type);

	virtual void insert(Model & model);
	virtual void update(Model & model);

protected:

	Expression * expression;
	PT::TextStream * out_stream; // IMPROVE ME give here an interface to the base stream (implement him)
	BaseEscaper * json_escaper;
	BaseEscaper * postgresql_escaper;

	bool expression_allocated;
	bool out_stream_allocated;
	bool json_escaper_allocated;
	bool postgresql_escaper_allocated;

	int assigned_db_type;

	void allocate_default_expression();
	void allocate_default_expression_if_needed();
	void deallocate_expression();

	void allocate_default_stream();
	void allocate_default_stream_if_needed();
	void deallocate_stream();

	void allocate_default_json_escaper();
	void allocate_default_json_escaper_if_needed();
	void deallocate_json_escaper();

	void allocate_default_postgresql_escaper();
	void allocate_default_postgresql_escaper_if_needed();
	void deallocate_postgresql_escaper();

	void allocate_default_db_escaper_if_needed();


	template<typename FieldValue>
	void field(const wchar_t * field_name, const FieldValue & field_value, bool insertable, bool updatable, bool is_primary_key)
	{
		if( expression )
		{
			expression->field(field_name, field_value, insertable, updatable, is_primary_key);
		}
	}

	/*
	 * Model is using field() method
	 */
	friend Model;
};

}

#endif
